import threading

from flask import Flask

from flask_discord_interactions import (
    DiscordInteractions,
    ResponseType,
    InteractionType,
    Message,
)


def test_flask():
    app = Flask(__name__)
    app.config["DONT_VALIDATE_SIGNATURE"] = True
    app.config["DONT_REGISTER_WITH_DISCORD"] = True

    discord = DiscordInteractions(app)

    @discord.command()
    def ping(ctx, pong: str = "ping"):
        return f"Ping {pong}!"

    discord.set_route("/interactions")

    with app.test_client() as client:
        response = client.post(
            "/interactions",
            json={
                "type": InteractionType.APPLICATION_COMMAND,
                "id": 1,
                "channel_id": "",
                "guild_id": "",
                "token": "",
                "data": {
                    "id": 1,
                    "name": "ping",
                    "options": [{"type": 1, "name": "Pong"}],
                },
                "member": {"id": 1, "nick": "", "user": {"id": 1, "username": "test"}},
            },
        )

        assert response.status_code == 200
        assert response.get_json()["type"] == ResponseType.CHANNEL_MESSAGE_WITH_SOURCE

        assert response.get_json()["data"]["content"] == "Ping Pong!"


def test_app_factory():
    discord = DiscordInteractions()

    @discord.command()
    def ping(ctx, pong: str = "ping"):
        return f"Ping {pong}!"

    def create_app():
        app = Flask(__name__)
        discord.init_app(app)
        discord.set_route("/interactions", app=app)
        return app

    app = create_app()
    app.config["DONT_VALIDATE_SIGNATURE"] = True
    app.config["DONT_REGISTER_WITH_DISCORD"] = True

    with app.test_client() as client:
        response = client.post(
            "/interactions",
            json={
                "type": InteractionType.APPLICATION_COMMAND,
                "id": 1,
                "channel_id": "",
                "guild_id": "",
                "token": "",
                "data": {
                    "id": 1,
                    "name": "ping",
                    "options": [{"type": 1, "name": "Pong"}],
                },
                "member": {"id": 1, "nick": "", "user": {"id": 1, "username": "test"}},
            },
        )

        assert response.status_code == 200
        assert response.get_json()["type"] == ResponseType.CHANNEL_MESSAGE_WITH_SOURCE

        assert response.get_json()["data"]["content"] == "Ping Pong!"


def test_followup():
    app = Flask(__name__)
    app.config["DONT_VALIDATE_SIGNATURE"] = True
    app.config["DONT_REGISTER_WITH_DISCORD"] = True

    discord = DiscordInteractions(app)

    ref_to_thread = None

    @discord.command()
    def ping(ctx):
        nonlocal ref_to_thread

        def do_followup():
            ctx.edit("hi")

        ref_to_thread = thread = threading.Thread(target=do_followup)
        thread.start()

        return Message(deferred=True)

    discord.set_route("/interactions")

    with app.test_client() as client:
        response = client.post(
            "/interactions",
            json={
                "type": InteractionType.APPLICATION_COMMAND,
                "id": 1,
                "channel_id": "",
                "guild_id": "",
                "token": "",
                "data": {
                    "id": 1,
                    "name": "ping",
                },
            },
        )

    # Make sure we wait for the thread to complete
    ref_to_thread.join()



import pytest

from flask import Flask

from flask_discord_interactions import DiscordInteractions, Client


@pytest.fixture(scope="module")
def discord():
    app = Flask(__name__)
    return DiscordInteractions(app)


@pytest.fixture(scope="module")
def client(discord):
    return Client(discord)



import dataclasses

from flask_discord_interactions.models.utils import LoadableDataclass


class ChannelType:
    "Represents the different :class:`Channel` type integers."
    GUILD_TEXT = 0
    DM = 1
    GUILD_VOICE = 2
    GROUP_DM = 3
    GUILD_CATEGORY = 4
    GUILD_NEWS = 5
    GUILD_STORE = 6


@dataclasses.dataclass
class Channel(LoadableDataclass):
    """
    Represents a Channel in Discord. This includes voice channels, text
    channels, and channel categories.

    Attributes
    ----------
    id
        The unique ID (snowflake) of the channel.
    name
        The name of the channel.
    permissions
        The permissions integer of the invoking user in that channel.
    type
        The type of channel.
    """

    id: str = None
    name: str = None
    permissions: int = None
    type: int = None



import dataclasses
from typing import Optional

from flask_discord_interactions.models.utils import LoadableDataclass


@dataclasses.dataclass
class Attachment(LoadableDataclass):
    """
    Represents an attachment
    Attributes
    ----------
    id
        The id of the attachment
    filename
        Name of the attached file
    description
        Optional description of the file. Used for screen readers
    content_type
        The media type of the file
    size
        Size of the attached file in bytes
    url
        Url to fetch the file
    proxy_url
        Url to fetch the file from the discord media proxy
    height
        Optinal height if the file is an image
    width
        Optinal width if the file is an image
    ephemeral
        Set to true on ephemeral messages and when getting attachment as command option values
    """

    id: Optional[str] = None
    filename: Optional[str] = None
    description: Optional[str] = None
    content_type: Optional[str] = None
    size: Optional[int] = None
    url: Optional[str] = None
    proxy_url: Optional[str] = None
    height: Optional[int] = None
    width: Optional[int] = None
    ephemeral: Optional[bool] = None



from dataclasses import dataclass
from typing import Any, Optional, Union

from flask_discord_interactions.models import User, Member, Channel, Role


class CommandOptionType:
    "Represents the different option type integers."
    SUB_COMMAND = 1
    SUB_COMMAND_GROUP = 2
    STRING = 3
    INTEGER = 4
    BOOLEAN = 5
    USER = 6
    CHANNEL = 7
    ROLE = 8
    MENTIONABLE = 9
    NUMBER = 10
    ATTACHMENT = 11


@dataclass
class Option:
    """
    Represents an option provided to a slash command.

    Attributes
    ----------
    name
        The name of the option.
    name_localizations
        A dict of localizations for the name of the option.
    type
        The type of the option. Provide either a value of
        :class:`.CommandOptionType` or a type (e.g. ``str``).
    description
        The description of the option. Defaults to "No description."
    description_localizations
        A dict of localizations for the description of the option.
    required
        Whether the option is required. Defaults to ``False``.
    options:
        A list of further options if the option is a subcommand or a subcommand group.
    choices
        A list of choices for the option.
    channel_types:
        A list of :class:`.ChannelType` for the option.
    min_value
        The minimum value of the option if the option is numeric.
    max_value
        The maximum value of the option if the option is numeric.
    autocomplete
        Whether the option should be autocompleted. Defaults to ``False``.
        Set to ``True`` if you have an autocomplete handler for this command.
    value
        Only present on incoming options passed to autocomplete objects. You
        shouldn't set this yourself. Represents the value that the user is
        currently typing.
    focused
        Only present on incoming options passed to autocomplete objects. True
        if the user is currently typing this option.
    """

    name: str
    type: int

    description: str = "No description"
    required: bool = False
    options: Optional[list] = None
    choices: Optional[list] = None
    channel_types: Optional[list] = None
    min_value: Optional[int] = None
    max_value: Optional[int] = None
    name_localizations: Optional[dict] = None
    description_localizations: Optional[dict] = None

    autocomplete: bool = False

    value: Any = None
    focused: Optional[bool] = None

    def __post_init__(self):
        if isinstance(self.type, type):
            if self.type == str:
                self.type = CommandOptionType.STRING
            elif self.type == int:
                self.type = CommandOptionType.INTEGER
            elif self.type == bool:
                self.type = CommandOptionType.BOOLEAN
            elif self.type in [User, Member]:
                self.type = CommandOptionType.USER
            elif self.type == Channel:
                self.type = CommandOptionType.CHANNEL
            elif self.type == Role:
                self.type = CommandOptionType.ROLE
            elif self.type == float:
                self.type = CommandOptionType.NUMBER
            else:
                raise ValueError(f"Unknown type {self.type}")

    @classmethod
    def from_data(cls, data):
        "Load this option from incoming Interaction data."
        return cls(
            name=data["name"],
            type=data["type"],
            value=data.get("value"),
            focused=data.get("focused"),
        )

    def dump(self):
        "Return this option as as a dict for registration with Discord."
        data = {
            "name": self.name,
            "name_localizations": self.name_localizations,
            "type": self.type,
            "description": self.description,
            "description_localizations": self.description_localizations,
            "required": self.required,
            "options": self.options,
            "channel_types": self.channel_types,
            "min_value": self.min_value,
            "max_value": self.max_value,
            "autocomplete": self.autocomplete,
        }
        if self.choices is not None:
            data["choices"] = [(c.dump() if isinstance(c, Choice) else c) for c in self.choices]
        return data


@dataclass
class Choice:
    """
    Represents an option choice. These can be directly set in the command or returned as autocomplete results
    name
        Name of the choice
    value
        Value of the choice
    name_localizations
        A dict of localizations for the name of the choice.
    """

    name: str
    value: Union[str, int]
    name_localizations: Optional[dict] = None

    def dump(self):
        "Return this choice as a dict for registration with Discord."
        data = {
            "name": self.name,
            "value": self.value,
            "name_localizations": self.name_localizations,
        }
        return data



def test_ping(discord, client):
    @discord.command()
    def ping(ctx, pong: str = "pong"):
        "Respond with a friendly 'pong'!"
        return f"Pong {pong}!"

    assert client.run("ping").content == "Pong pong!"
    assert client.run("ping", pong="sprong").content == "Pong sprong!"



from dataclasses import dataclass, asdict
from typing import List, Union


class ComponentType:
    ACTION_ROW = 1
    BUTTON = 2
    SELECT_MENU = 3
    TEXT_INPUT = 4


class Component:
    "Represents a Message Component."

    def dump(self):
        "Returns this Component as a dictionary, removing fields which are None."

        def filter_none(d):
            if isinstance(d, dict):
                return {k: filter_none(v) for k, v in d.items() if v is not None}
            elif isinstance(d, list):
                return [filter_none(v) for v in d if v is not None]
            else:
                return d

        return filter_none(asdict(self))

    @staticmethod
    def from_dict(data):
        if data.get("type") == ComponentType.ACTION_ROW:
            return ActionRow.from_dict(data)
        elif data.get("type") == ComponentType.BUTTON:
            return Button(**data)
        elif data.get("type") == ComponentType.SELECT_MENU:
            return SelectMenu(**data)
        elif data.get("type") == ComponentType.TEXT_INPUT:
            return TextInput(**data)
        else:
            raise ValueError(f"Unknown component type: {data.get('type')}")


@dataclass
class CustomIdComponent(Component):
    """
    Represents a Message Component with a Custom ID.

    Attributes
    ----------
    custom_id: str | List
        The custom ID of the component. Strings represent a single ID, lists
        take advantage of newlines to store state alongside the ID.
    """

    custom_id: Union[str, List] = None

    def __post_init__(self):
        if isinstance(self.custom_id, list) or isinstance(self.custom_id, tuple):
            self.custom_id = "\n".join(str(item) for item in self.custom_id)

        if self.custom_id and len(self.custom_id) > 100:
            raise ValueError("custom_id has maximum 100 characters")


@dataclass
class ActionRow(Component):
    """
    Represents an ActionRow message component.

    Attributes
    ----------
    components: List[Component]
        The message components to display in the action row.
        Limited to any of the following:
        - 5 Buttons
        - 1 Select Menu
    """

    components: List[CustomIdComponent] = None

    type: int = ComponentType.ACTION_ROW

    @staticmethod
    def from_dict(data):
        components = [
            Component.from_dict(component) for component in data.get("components", [])
        ]
        return ActionRow(components=components)

    def __post_init__(self):
        if self.components:
            if len(self.components) > 5:
                raise ValueError("ActionRow can have at most 5 components")

            for component in self.components:
                if component.type == ComponentType.ACTION_ROW:
                    raise ValueError("nested action rows not allowed")
                elif component.type == ComponentType.BUTTON:
                    pass
                elif component.type == ComponentType.SELECT_MENU:
                    if len(self.components) > 1:
                        raise ValueError(
                            "select menu must be the only child of action row"
                        )
                elif component.type == ComponentType.TEXT_INPUT:
                    if len(self.components) > 1:
                        raise ValueError(
                            "text input must be the only child of action row"
                        )


class ButtonStyles:
    "Represents the styles that can be applied to Button message components."
    PRIMARY = 1
    SECONDARY = 2
    SUCCESS = 3
    DANGER = 4
    LINK = 5


@dataclass
class Button(CustomIdComponent):
    """
    Represents a Button message component.

    Attributes
    ----------
    style: int
        The style of the button (see :class:`ButtonStyles`).
    label: str
        The label displayed on the button.
    url: str
        For link buttons, the URL that the button links to.
    disabled: bool
        Whether the button is disabled.
    """

    style: int = ButtonStyles.PRIMARY
    label: str = None

    emoji: dict = None
    url: str = None
    disabled: bool = False

    type: int = ComponentType.BUTTON

    def __post_init__(self):
        super().__post_init__()

        if self.style == ButtonStyles.LINK:
            if self.url is None:
                raise ValueError("Link buttons require a url")
        else:
            if self.custom_id is None:
                raise ValueError("Buttons require custom_id")

        if isinstance(self.custom_id, list) or isinstance(self.custom_id, tuple):
            self.custom_id = "\n".join(str(item) for item in self.custom_id)

        if self.custom_id and len(self.custom_id) > 100:
            raise ValueError("custom_id has maximum 100 characters")


@dataclass
class SelectMenuOption:
    """
    Represents an option in a SelectMenu message component.

    Attributes
    ----------
    label: str
        The label displayed on the option.
    value: str
        The value of the option passed to the custom ID handler.
    description: str
        The description displayed on the option.
    emoji: dict
        The emoji displayed on the option.
    default: bool
        Whether the option is the default option.
    """

    label: str
    value: str

    description: str = None
    emoji: dict = None
    default: bool = False


@dataclass
class SelectMenu(CustomIdComponent):
    """
    Represents a SelectMenu message component.

    Attributes
    ----------
    options: List[SelectMenuOption]
        The options to display in the select menu.
    placeholder: str
        The placeholder displayed when the select menu is empty.
    min_values: int
        The minimum number of options that must be selected.
    max_values: int
        The maximum number of options that can be selected.
    disabled: bool
        Whether the select menu is disabled.
    values: list
        Selected options. Only present when receiving components from a modal.
    """

    options: List[SelectMenuOption] = None

    placeholder: str = None
    min_values: int = 1
    max_values: int = 1
    disabled: bool = False
    values: list = None

    type: int = ComponentType.SELECT_MENU

    def __post_init__(self):
        super().__post_init__()

        if self.options and len(self.options) > 25:
            raise ValueError("Select is limited to 25 options")

        if self.placeholder and len(self.placeholder) > 100:
            raise ValueError("Placeholder is max 100 characters")

        if self.min_values > self.max_values:
            raise ValueError("min_values must be less than or equal to max_values")

        if self.max_values > 25:
            raise ValueError("max_values is maximum 25")


class TextStyles:
    "Represents the styles that can be applied to TextInput modal components."
    SHORT = 1
    PARAGRAPH = 2


@dataclass
class TextInput(CustomIdComponent):
    """
    Represents a TextInput modal component.

    Attributes
    ----------
    label: str
        The label displayed for the field.
    style: int
        The style of the text input (see :class:`TextStyles`).
    value: str
        A pre-filled value for this component, max 4000 characters.
    placeholder: str
        The placeholder displayed when the select menu is empty.
    min_length: int
        The minimum input length for a text input, min 0, max 4000
    max_length: int
        The maximum input length for a text input, min 1, max 4000
    required: bool
        Whether the text input is required.
    """

    label: str = None

    style: int = TextStyles.SHORT
    value: str = None
    placeholder: str = None

    min_length: int = 0
    max_length: int = 4000

    required: bool = True

    type: int = ComponentType.TEXT_INPUT

    def __post_init__(self):
        super().__post_init__()

        if self.min_length > self.max_length:
            raise ValueError("min_length must be less than or equal to max_length")

        if not 0 <= self.min_length <= 4000:
            raise ValueError("max_length must be between 0 and 4000, inclusive")

        if not 0 < self.max_length <= 4000:
            raise ValueError("max_length must be between 1 and 4000, inclusive")



from dataclasses import dataclass, asdict
from typing import List

from flask_discord_interactions.models.utils import LoadableDataclass


@dataclass
class Footer(LoadableDataclass):
    "Represents the footer of an Embed."
    text: str
    icon_url: str = None
    proxy_icon_url: str = None


@dataclass
class Field(LoadableDataclass):
    "Represents a field on an Embed."
    name: str
    value: str
    inline: bool = False


@dataclass
class Media(LoadableDataclass):
    "Represents a thumbnail, image, or video on an Embed."
    url: str = None
    proxy_url: str = None
    height: int = None
    width: int = None


@dataclass
class Provider(LoadableDataclass):
    "Represents a provider of an Embed."
    name: str = None
    url: str = None


@dataclass
class Author(LoadableDataclass):
    "Represents an author of an embed."
    name: str = None
    url: str = None
    icon_url: str = None
    proxy_icon_url: str = None


@dataclass
class Embed(LoadableDataclass):
    """
    Represents an Embed to be sent as part of a Message.

    Attributes
    ----------
    title
        The title of the embed.
    description
        The description in the embed.
    url
        The URL that the embed title links to.
    timestamp
        An ISO8601 timestamp included in the embed.
    color
        An integer representing the color of the sidebar of the embed.
    footer
        A :class:`Footer` representing the footer of the embed.
    image
        A :class:`Media` representing the image of the embed.
    thumbnail
        A :class:`Media` representing the thumbnail of the embed.
    video
        A :class:`Media` representing the video of the embed.
    provider
        A :class:`Provider` representing the name and URL of the provider of
        the embed.
    author
        A :class:`Author` representing the author of the embed.
    fields
        A list of :class:`Field` objects representing the fields of the embed.
    """

    title: str = None
    description: str = None
    url: str = None
    timestamp: str = None
    color: int = None
    footer: Footer = None
    image: Media = None
    thumbnail: Media = None
    video: Media = None
    provider: Provider = None
    author: Author = None
    fields: List[Field] = None

    def dump(self):
        "Returns this Embed as a dictionary, removing fields which are None."

        def filter_none(d):
            if isinstance(d, dict):
                return {k: filter_none(v) for k, v in d.items() if v is not None}
            else:
                return d

        return filter_none(asdict(self))



import json

import pytest

from flask_discord_interactions import Message, ResponseType, Embed, embed


def test_content(discord, client):
    @discord.command()
    def content(ctx):
        return "quality content"

    assert client.run("content").content == "quality content"


def test_dict_embed(discord, client):
    my_embed = {"title": "My Cool Embed"}

    @discord.command()
    def embeddy(ctx):
        return Message(embed=my_embed)

    assert client.run("embeddy").dump_embeds() == [my_embed]

    @discord.command()
    def multiple(ctx):
        return Message(embeds=[my_embed, my_embed])

    assert client.run("multiple").dump_embeds() == [my_embed, my_embed]


def test_class_embed(discord, client):
    my_embed = Embed(title="My Cool Embed", image=embed.Media(url="https://google.com"))

    output = {"title": "My Cool Embed", "image": {"url": "https://google.com"}}

    @discord.command()
    def embeddy(ctx):
        return Message(embed=my_embed)

    assert client.run("embeddy").dump_embeds() == [output]

    @discord.command()
    def multiple(ctx):
        return Message(embeds=[my_embed, my_embed])

    assert client.run("multiple").dump_embeds() == [output, output]


def test_improper_immediate(discord, client):
    fp = open("README.md", "r")

    @discord.command()
    def improper_immediate(ctx):
        return Message(file=("README.md", fp, "text/markdown"))

    result = client.run("improper_immediate")

    with pytest.raises(ValueError):
        result.dump()

    fp.close()


def test_improper_followup(discord, client):
    @discord.command()
    def improper_followup(ctx):
        return Message("hi", ephemeral=True)

    result = client.run("improper_followup")

    with pytest.raises(ValueError):
        result.dump_followup()


def test_ephemeral(discord, client):
    @discord.command()
    def ephemeral(ctx):
        return Message("hi", ephemeral=True)

    assert client.run("ephemeral").content == "hi"
    assert client.run("ephemeral").flags == 64


def test_dump_immediate(discord, client):
    @discord.command()
    def use_response(ctx):
        return Message("hi", tts=True, embed=Embed(title="hello!"))

    expected = {
        "type": ResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
        "data": {
            "content": "hi",
            "embeds": [{"title": "hello!"}],
            "flags": 0,
            "tts": True,
            "allowed_mentions": {"parse": ["roles", "users", "everyone"]},
            "components": None,
        },
    }

    assert client.run("use_response").dump() == expected


def test_dump_followup():
    resp = Message(
        content="Followup",
        embed=Embed(title="hello!"),
    )

    expected = {
        "content": "Followup",
        "embeds": [{"title": "hello!"}],
        "tts": False,
        "allowed_mentions": {"parse": ["roles", "users", "everyone"]},
        "components": None,
    }

    assert resp.dump_followup() == expected


def test_dump_multipart():
    fp = open("README.md", "r")
    resp = Message(
        content="Followup",
        embed=Embed(title="hello!"),
        file=("README.md", fp, "text/markdown"),
    )

    expected = {
        "data": {
            "payload_json": json.dumps(
                {
                    "content": "Followup",
                    "tts": False,
                    "embeds": [{"title": "hello!"}],
                    "allowed_mentions": {"parse": ["roles", "users", "everyone"]},
                    "components": None,
                }
            )
        },
        "files": [("file", ("README.md", fp, "text/markdown"))],
    }

    try:
        assert resp.dump_multipart() == expected
    finally:
        fp.close()



from setuptools import setup, find_packages


with open("README.md") as file:
    readme = file.read()


setup(
    name="Flask-Discord-Interactions",
    version="1.7.0",
    url="https://github.com/Breq16/flask-discord-interactions",
    author="Brooke Chalmers",
    author_email="breq@breq.dev",
    description="A Flask extension for Discord slash commands.",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms="any",
    install_requires=["Flask", "requests", "PyNaCl"],
    extras_require={"async": ["Quart", "aiohttp"]},
    tests_require=["pytest"],
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
    ],
    python_requires=">=3.7",
)



from dataclasses import dataclass
from typing import List, Union

from flask_discord_interactions.models.utils import LoadableDataclass
from flask_discord_interactions.models.component import Component, ComponentType
from flask_discord_interactions.enums import ResponseType


@dataclass
class Modal(LoadableDataclass):
    """
    Represents a Modal form window.

    Attributes
    ----------
    custom_id
        The ID for Callbacks.
    title
        The title of the modal.
    components
        A list of :class:`ActionRow` objects representing the rows of the form.
        Must have at least 1 row, at most 5 rows.
    """

    custom_id: Union[str, list] = None
    title: str = None
    components: List[Component] = None

    def __post_init__(self):
        # Verify Custom ID
        if self.custom_id is None:
            raise ValueError("Modals require custom_id")

        if isinstance(self.custom_id, list) or isinstance(self.custom_id, tuple):
            self.custom_id = "\n".join(str(item) for item in self.custom_id)

        if len(self.custom_id) > 100:
            raise ValueError("custom_id has maximum 100 characters")

        # Verify Title
        if self.title is None:
            raise ValueError("Modals require title")

        # Verify Components
        if not (
            self.components
            and isinstance(self.components, (list, tuple, set))
            and 1 <= len(self.components) <= 5
        ):
            raise ValueError(
                "Modal must have between 1 and 5 (inclusive) components that make up the modal"
            )

        for component in self.components:
            if component.type != ComponentType.ACTION_ROW:
                raise ValueError("Only Action Row components are supported for Modals")

    def dump_components(self):
        "Returns the message components as a list of dicts."
        return [c.dump() for c in self.components]

    def dump(self):
        """
        Return this ``Modal`` as a dict to be sent in response to an
        incoming webhook.
        """
        return {
            "type": ResponseType.MODAL,
            "data": {
                "custom_id": self.custom_id,
                "title": self.title,
                "components": self.dump_components(),
            },
        }

    def dump_handler(self):
        "Return a modal in component handlers."
        return self.dump()
